#define LIGHT_SENSOR_PIN PD7
#define WATER_SENSOR_PIN A5
#define SERVOPIN 8

/* This function will rotate the servo for a set amount of time */
void servo(byte pin, int speed, unsigned int time)
{
  if(speed > 500)
    speed = 500;
  if(speed < -500)
    speed = -500;

  time = time / 20;
  for(time; time > 0; time--)
  {
    servoPulse(pin, speed);
  }
}

/* Send a 1 duty cycle (20ms) pulse to the servo with the required on time*/
void servoPulse(byte pin, int speed)
{
  digitalWrite(pin,HIGH);
  delayMicroseconds(1500 + speed);
  digitalWrite(pin,LOW);
  delayMicroseconds(500 - speed);

  delay(18);
}

bool closed = false;
volatile bool sun_detected = false;
volatile bool change_status = false;
bool rain_detected = false;

ISR(PCINT2_vect) { //only PD7 will generate this type of interrupt so no need for differentiate it 
  // cod întrerupere de tip pin change

  int sun_value = digitalRead(LIGHT_SENSOR_PIN);

  if(sun_value == 0) //sun is detected
  {
    sun_detected = true;
    change_status = true;
  }

  if(sun_value == 1) //sun is not detected
  {
    sun_detected = false;
    change_status = true;
  }
}

bool manual_sun_sensor = false;
bool manual_rain_sensor = false;

void setup() {
  // put your setup code here, to run once:
  cli();
  PCICR |= 0b00000100; //turn on port D
  PCMSK2 |= (1 << PCINT23); //PD7 is pin change interrupt
  sei();
  pinMode(SERVOPIN, OUTPUT);
  Serial.begin(9600);

  Serial.println("Hello!");
  Serial.println("===Configure Sunlight Detector===");
  Serial.println("Type 1 for automated opening/closing when direct sunlight is detected.");
  Serial.println("Type 2 for manual opening/closing when direct sunlight is detected.");

  while(true)
  {
    if(Serial.available() > 0)
    {
      int value = Serial.read();

      if(value == '2')
      {
        manual_sun_sensor = true;
      }

      break;
    }
  }

  Serial.println("OK.");
  Serial.println("===Configure Rain Detector===");
  Serial.println("Type 1 for automated opening/closing when rain is detected.");
  Serial.println("Type 2 for manual opening/closing when rain is detected.");

  while(true)
  {
    if(Serial.available() > 0)
    {
      int value = Serial.read();

      if(value == '2')
      {
        manual_rain_sensor = true;
      }

      break;
    }
  }

  Serial.println("OK.");
  Serial.print("Sun is ");
  if(manual_sun_sensor == true)
  {
    Serial.println("manually configured.");
  }
  else
  {
    Serial.println("automated configured.");
  }

  Serial.print("Rain is ");
  if(manual_rain_sensor == true)
  {
    Serial.println("manually configured.");
  }
  else
  {
    Serial.println("automated configured.");
  }

  Serial.println("All done!");
}

bool change_status_rain = false;

int sun_closed = 0; // 0-default 1-sun 2-rain

void check_rain()
{
  int water_level; //if water is more than 500 then its raining
  water_level = analogRead(WATER_SENSOR_PIN);

  if(water_level > 300) //rain is detected
  {
    rain_detected = true;

    if(manual_rain_sensor == false) //automated
    {
      if(closed == false)
      {
        Serial.println("Rain detected: closing shader.");
        sun_closed = 2;
        servo(SERVOPIN, 250, 2000);
        closed = true;
      }
    }
    else //manual
    {
      if(change_status_rain == false)
      {
        change_status_rain = true;
        Serial.println("Rain detected!");

        if(closed == false)
        {
          Serial.print("Current status: opened, rain detected, ");
          if(sun_detected == true)
          {
            Serial.println("sunlight detected.");
          }
          else
          {
            Serial.println("sunlight not detected.");
          }
          Serial.println("Type 1 to close or 2 to keep it open.");
          while(true)
          {
            if(Serial.available() > 0)
            {
              int value = Serial.read();

              if(value == '1')
              {
                sun_closed = 2;
                servo(SERVOPIN, 250, 2000);
                closed = true;
              }
              break;
            }
          }

          Serial.println("OK.");
        }
        else
        {
          Serial.print("Current status: closed, rain detected, ");
          if(sun_detected == true)
          {
            Serial.println("sunlight detected.");
          }
          else
          {
            Serial.println("sunlight not detected.");
          }
          Serial.println("Type 1 to open or 2 to keep it closed.");
          while(true)
          {
            if(Serial.available() > 0)
            {
              int value = Serial.read();

              if(value == '1')
              {
                servo(SERVOPIN, -250, 2000);
                closed = false;
              }
              break;
            }
          }

          Serial.println("OK.");
        }
      }
    }
  }
  else //rain stopped
  {
    rain_detected = false;

    if(manual_rain_sensor == false)
    {
      if((closed == true && sun_detected == false && manual_sun_sensor == false && (sun_closed == 0 || sun_closed == 2)) || (manual_sun_sensor == true && closed == true && sun_closed != 1))
      {
        Serial.println("Rain no longer detected: opening shader.");
        sun_closed = 0;
        servo(SERVOPIN, -250, 2000);
        closed = false;
      }
    }
    else
    {
      if(change_status_rain == true)
      {
        change_status_rain = false;
        Serial.println("Rain not detected.");

        if(closed == false)
        {
          Serial.print("Current status: opened, rain not detected, ");
          if(sun_detected == true)
          {
            Serial.println("sunlight detected.");
          }
          else
          {
            Serial.println("sunlight not detected.");
          }
          Serial.println("Type 1 to close or 2 to keep it open.");
          while(true)
          {
            if(Serial.available() > 0)
            {
              int value = Serial.read();

              if(value == '1')
              {
                servo(SERVOPIN, 250, 2000);
                closed = true;
              }
              break;
            }
          }

          Serial.println("OK.");
        }
        else
        {
          Serial.print("Current status: closed, rain not detected, ");
          if(sun_detected == true)
          {
            Serial.println("sunlight detected.");
          }
          else
          {
            Serial.println("sunlight not detected.");
          }
          Serial.println("Type 1 to open or 2 to keep it closed.");
          while(true)
          {
            if(Serial.available() > 0)
            {
              int value = Serial.read();

              if(value == '1')
              {
                servo(SERVOPIN, -250, 2000);
                closed = false;
              }
              break;
            }
          }

          Serial.println("OK.");
        }
      }
    }
  }
}

void check_sun()
{
  if(sun_detected == true)
  {
    if(manual_sun_sensor == false)
    {
      if(closed == false)
      {
        Serial.println("Sun detected: closing shader.");
        sun_closed = 1;
        servo(SERVOPIN, 250, 2000);
        closed = true;
      }
    }
    else
    {
      if(change_status == true && sun_detected == true)
      {
        change_status = false;

        Serial.println("Sun detected!");

        if(closed == false)
        {
          Serial.print("Current status: opened, sunlight detected, ");
          if(rain_detected == true)
          {
            Serial.println("rain detected.");
          }
          else
          {
            Serial.println("rain not detected.");
          }
          Serial.println("Type 1 to close or 2 to keep it open.");
          while(true)
          {
            if(Serial.available() > 0)
            {
              int value = Serial.read();

              if(value == '1')
              {
                sun_closed = 1;
                servo(SERVOPIN, 250, 2000);
                closed = true;
              }
              break;
            }
          }

          Serial.println("OK.");
        }
        else
        {
          Serial.print("Current status: closed, sunlight detected, ");
          if(rain_detected == true)
          {
            Serial.println("rain detected.");
          }
          else
          {
            Serial.println("rain not detected.");
          }
          Serial.println("Type 1 to open or 2 to keep it closed.");
          while(true)
          {
            if(Serial.available() > 0)
            {
              int value = Serial.read();

              if(value == '1')
              {
                servo(SERVOPIN, -250, 2000);
                closed = false;
              }
              break;
            }
          }

          Serial.println("OK.");
        }
      }
    }
  }
  else //sun stopped
  {
    if(manual_sun_sensor == false)
    {
      if((closed == true && rain_detected == false && manual_rain_sensor == false && (sun_closed == 0 || sun_closed == 1)) || (manual_rain_sensor == true && closed == true && sun_closed != 2)) 
      {
        Serial.println("Sun no longer detected: opening shader.");
        sun_closed = 0;
        servo(SERVOPIN, -250, 2000);
        closed = false;
      }
    }
    else
    {
      if(change_status == true && sun_detected == false)
      {
        change_status = false;

        Serial.println("Sun not detected.");

        if(closed == false)
        {
          Serial.print("Current status: opened, sunlight not detected, ");
          if(rain_detected == true)
          {
            Serial.println("rain detected.");
          }
          else
          {
            Serial.println("rain not detected.");
          }
          Serial.println("Type 1 to close or 2 to keep it open.");
          while(true)
          {
            if(Serial.available() > 0)
            {
              int value = Serial.read();

              if(value == '1')
              {
                servo(SERVOPIN, 250, 2000);
                closed = true;
              }
              break;
            }
          }

          Serial.println("OK.");
        }
        else
        {
          Serial.print("Current status: closed, sunlight not detected, ");
          if(rain_detected == true)
          {
            Serial.println("rain detected.");
          }
          else
          {
            Serial.println("rain not detected.");
          }
          Serial.println("Type 1 to open or 2 to keep it closed.");
          while(true)
          {
            if(Serial.available() > 0)
            {
              int value = Serial.read();

              if(value == '1')
              {
                servo(SERVOPIN, -250, 2000);
                closed = false;
              }
              break;
            }
          }

          Serial.println("OK.");
        }
      }
    }
  }
}

void loop() {
  check_rain();
  check_sun();
}
